unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, DB, DBClient, XPMan, StdCtrls, ActnCtrls, XPStyleActnCtrls, ActnList,
  ActnMan, ToolWin, ActnMenus, Mask, DBCtrls, ComCtrls, ImgList, MidasLib;

type
  TForm1 = class(TForm)
    DataSource1: TDataSource;
    CDS: TClientDataSet;
    XPManifest: TXPManifest;
    ActionMainMenuBar: TActionMainMenuBar;
    ActionManager: TActionManager;
    ActionToolBar: TActionToolBar;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    MovieList: TListBox;
    FindEdit: TEdit;
    MovieIDLabel: TLabel;
    MovieNameLabel: TLabel;
    DBMovieName: TDBEdit;
    DBMovieID: TDBEdit;
    StatusBar: TStatusBar;
    NewAction: TAction;
    OpenAction: TAction;
    SaveAction: TAction;
    SaveAsAction: TAction;
    ExitAction: TAction;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    ImageList: TImageList;
    AddAction: TAction;
    DeleteAction: TAction;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FindEditChange(Sender: TObject);
    procedure DeleteActionUpdate(Sender: TObject);
    procedure DeleteActionExecute(Sender: TObject);
    procedure MovieListClick(Sender: TObject);
    procedure AddActionExecute(Sender: TObject);
    procedure ExitActionExecute(Sender: TObject);
    procedure SaveAsActionExecute(Sender: TObject);
    procedure SaveActionExecute(Sender: TObject);
    procedure OpenActionExecute(Sender: TObject);
    procedure NewActionExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure DisplayDataset;
    procedure OpenCatalog(const AFileName: string);
    procedure SaveCatalog(const AFileName: string);
  end;

var
  Form1: TForm1;

implementation

uses AddMovie;

{$R *.dfm}

procedure TForm1.DisplayDataset;
begin
  MovieList.Clear;
  MovieList.Items.BeginUpdate;

  CDS.First;
  while not CDS.Eof do
  begin
    MovieList.Items.Add(CDS.FieldByName('MovieName').AsString);
    CDS.Next;
  end;

  MovieList.Items.EndUpdate;

  if MovieList.Items.Count = 0 then
    StatusBar.SimpleText := '  .'
  else
    StatusBar.SimpleText := '   : ' +
      IntToStr(MovieList.Items.Count);
end;

procedure TForm1.OpenCatalog(const AFileName: string);
begin
  CDS.Close;
  CDS.FileName := AFileName;

  {   CreateDataSet     . }
  if not FileExists(AFileName) then CDS.CreateDataSet;
  CDS.Open;

  {   DisplayDataSet   
            . }
  DisplayDataset;
end;

procedure TForm1.SaveCatalog(const AFileName: string);
begin
  CDS.FileName := AFileName;
   {   MergeChangeLog,     ,
          
      OnCloseQuery. }
  CDS.MergeChangeLog;
  CDS.SaveToFile(CDS.FileName, dfXML);
end;

procedure TForm1.NewActionExecute(Sender: TObject);
begin
  if OpenDialog1.Execute then
    OpenCatalog(OpenDialog1.FileName);
end;

procedure TForm1.OpenActionExecute(Sender: TObject);
begin
  if OpenDialog1.Execute then
    OpenCatalog(OpenDialog1.FileName);
end;

procedure TForm1.SaveActionExecute(Sender: TObject);
begin
  if CDS.FileName <> '' then
    SaveCatalog(CDS.FileName)
  else
    SaveAsAction.Execute;
end;

procedure TForm1.SaveAsActionExecute(Sender: TObject);
begin
  if SaveDialog1.Execute then
    SaveCatalog(SaveDialog1.FileName);
end;

procedure TForm1.ExitActionExecute(Sender: TObject);
begin
  Close;
end;

procedure TForm1.AddActionExecute(Sender: TObject);
begin
  with TAddMovieForm.Create(Self) do
  begin
    MovieID.Text := IntToStr(Succ(MovieList.Items.Count));
    ShowModal;

    if ModalResult = mrOK then
    begin
      CDS.AppendRecord([StrToInt(MovieID.Text), MovieName.Text]);
      FindEdit.Clear; {  ,    . }

      {   . }
      DisplayDataset;
    end; //   if ModalResult.
    Free;
  end; //   with.
end;

procedure TForm1.MovieListClick(Sender: TObject);
begin
  if MovieList.ItemIndex <> -1 then
    CDS.Locate('MovieName', MovieList.Items[MovieList.ItemIndex], []);
end;

procedure TForm1.DeleteActionExecute(Sender: TObject);
const
  DELETE_MOVIE = '    :'#13;
begin
  if MessageDlg(DELETE_MOVIE + MovieList.Items[MovieList.ItemIndex] + '?',
    mtConfirmation, mbYesNo, 0) = mrYes then
  begin
    CDS.Delete;
    DisplayDataset;
  end; //   if MessageDlg.
end;

procedure TForm1.DeleteActionUpdate(Sender: TObject);
begin
  DeleteAction.Enabled := MovieList.ItemIndex <> -1;
end;

procedure TForm1.FindEditChange(Sender: TObject);
const
  SUBSTRING = 'Substring(MovieName, 1, %d) = ''%s''';
var
  selectedMovie: string;
begin
  CDS.Filter := Format(SUBSTRING, [Length(FindEdit.Text), FindEdit.Text]);

  {  ,     FindEdit  . }
  CDS.Filtered := FindEdit.Text <> '';

  {   DisplayDataSet,     . }
  DisplayDataSet;

  selectedMovie := CDS.FieldByName('MovieName').AsString;
  MovieList.ItemIndex := MovieList.Items.IndexOf(selectedMovie);
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if CDS.ChangeCount > 0 then
  begin
    case MessageDlg('    ?',
      mtConfirmation, mbYesNoCancel, 0) of
        mrYes: SaveAction.Execute;
        mrNo: CDS.CancelUpdates;
        mrCancel: CanClose := False;
    end; //   case.
  end; //   if.
end;

end.
